package port

import (
	"net/http"

	"gitea.dwysokinski.me/twhelp/corev3/internal/domain"
	"gitea.dwysokinski.me/twhelp/corev3/internal/port/internal/apimodel"
)

func (h *apiHTTPHandler) ListServers(
	w http.ResponseWriter,
	r *http.Request,
	versionCode apimodel.VersionCodePathParam,
	params apimodel.ListServersParams,
) {
	domainParams := domain.NewListServersParams()

	if err := domainParams.SetSort([]domain.ServerSort{domain.ServerSortOpenDESC, domain.ServerSortKeyASC}); err != nil {
		apiErrorRenderer{errors: []error{err}, formatErrorPath: formatListServersParamsErrorPath}.render(w, r)
		return
	}

	if err := domainParams.SetVersionCodes([]string{versionCode}); err != nil {
		apiErrorRenderer{errors: []error{err}, formatErrorPath: formatListServersParamsErrorPath}.render(w, r)
		return
	}

	if params.Open != nil {
		if err := domainParams.SetOpen(domain.NullBool{
			Value: *params.Open,
			Valid: true,
		}); err != nil {
			apiErrorRenderer{errors: []error{err}, formatErrorPath: formatListServersParamsErrorPath}.render(w, r)
			return
		}
	}

	if params.Limit != nil {
		if err := domainParams.SetLimit(*params.Limit); err != nil {
			apiErrorRenderer{errors: []error{err}, formatErrorPath: formatListServersParamsErrorPath}.render(w, r)
			return
		}
	}

	if params.Cursor != nil {
		if err := domainParams.SetEncodedCursor(*params.Cursor); err != nil {
			apiErrorRenderer{errors: []error{err}, formatErrorPath: formatListServersParamsErrorPath}.render(w, r)
			return
		}
	}

	res, err := h.serverSvc.List(r.Context(), domainParams)
	if err != nil {
		apiErrorRenderer{errors: []error{err}}.render(w, r)
		return
	}

	renderJSON(w, r, http.StatusOK, apimodel.NewListServersResponse(res))
}

func formatListServersParamsErrorPath(segments []errorPathSegment) []string {
	if segments[0].model != "ListServersParams" {
		return nil
	}

	switch segments[0].field {
	case "cursor":
		return []string{"$query", "cursor"}
	case "limit":
		return []string{"$query", "limit"}
	case "open":
		return []string{"$query", "open"}
	case "sort":
		return []string{"$query", "sort"}
	case "versionCodes":
		return []string{"$path", "versionCode"}
	default:
		return nil
	}
}
